unit Deck;

interface

const
  CardsInDeck = 52;
type
  TDeck = array[ 1..CardsInDeck ] of integer;
  PDeck = ^TDeck;

procedure deckMoveCard( var Deck: TDeck; PosFrom, PosTo: integer );
function deckFromString( Deck: string; Dest: PDeck ): integer;
function deckToString( const Deck: TDeck ): string;

implementation

uses
  Cards, SysUtils;

procedure deckMoveCard( var Deck: TDeck; PosFrom, PosTo: integer );
var
  C: integer;
  I: integer;
begin
  if( PosFrom = PosTo )or( PosTo < 1 )or( PosTo > CardsInDeck ) then
    Exit;

  C := Deck[PosFrom];
  if( PosTo < PosFrom ) then begin
    for I:=PosFrom downto PosTo+1 do
      Deck[I] := Deck[I-1];
  end
  else begin
    for I:=PosFrom to PosTo-1 do
      Deck[I] := Deck[I+1];
  end;
  Deck[PosTo] := C;
end;

function deckFromString( Deck: string; Dest: PDeck ): integer;
var
  Num, Value, Suit: integer;
  C: char;
  P: integer;
  D: TDeck;
begin
  Result := -1;

  Deck := Trim( Deck ) + ' ';

  Num := 1;
  Value := 0;
  Suit := 0;

  P := 1;
  while( P <= Length(Deck) ) do begin
    // Value
    C := UpCase(Deck[P]);
    Inc( P );
    case C of
      '2','3','4','5','6','7','8','9':
        Value := Ord(C) - Ord('0');
      'A':
        Value := 1;
      'J':
        Value := 11;
      'Q':
        Value := 12;
      'K':
        Value := 13;
      '1': begin
          if( Deck[P] <> '0' ) then begin
            Exit;
          end;
          Value := 10;
          Inc( P );
        end;
      else begin
        Exit;
      end;
    end;

    // Suit
    C := UpCase(Deck[P]);
    Inc( P );
    case C of
      'H': Suit := SuitHearts;
      'S': Suit := SuitSpades;
      'D': Suit := SuitDiamonds;
      'C': Suit := SuitClubs;
      else begin
        Exit;
      end;
    end;

    // Assign to card
    if( Num > 52 ) then begin
      Exit;
    end;
    D[Num] := cardMake( Value, Suit );
    Inc( Num );

    while( Deck[P] in [' '] ) do
      Inc( P );
  end;

  if( Num <> 53 ) then begin
    Exit;
  end;

  // Copy deck
  if( Dest <> nil ) then begin
    Dest^ := D;
  end;

  Result := 0;
end;

function deckToString( const Deck: TDeck ): string;
const
  ValueName: array[1..13] of string = ('A','2','3','4','5','6','7','8','9','10','J','Q','K');
var
  Value, Suit: integer;
  I: integer;
begin
  Result := '';
  for I:=1 to CardsInDeck do begin
    cardUnmake( Deck[I], Value, Suit );
    Result := Result + ValueName[Value];
    case Suit of
      SuitHearts: Result := Result + 'H ';
      SuitSpades: Result := Result + 'S ';
      SuitDiamonds: Result := Result + 'D ';
      SuitClubs: Result := Result + 'C ';
    end;
  end;
end;

end.
